
#ifndef __STIMAGE_H__
#define __STIMAGE_H__


#include "st.h"

/**
*	Representation of an image pixel
*/
class STPixel {
  public:
  unsigned char r;
  unsigned char g;
  unsigned char b;
  unsigned char a;
};


/**
* The STImage class encapsulates image pixel data.  Image data is stored
* as an array of STPixels.  Note that image data is stored in a format that
* is consistent with OpenGL: Row major with the bottom row of the image being
* first in the array of pixels. Eg. the bottom left pixel in the image is the
* first pixel in the internal array of pixels.
*/
class STImage : public STDrawable {

public:

  STImage();
  ~STImage();

  int GetHeight() const;
  int GetWidth() const;

  
  STStatus  Load(const char* pathname);
  STStatus  Save(const char* pathname);
  void		Read(int startx, int starty, int width, int height);
  STPixel*  Pixels();
  void      Draw();

private:

  int mHeight;
  int mWidth;
  
  STPixel* mPixels;

  STStatus  LoadPPM(const char* pathname);
  STStatus  LoadPNG(const char* pathname);
  STStatus  LoadJPG(const char* pathname);

  STStatus  SavePPM(const char* pathname);
  STStatus  SavePNG(const char* pathname);
  STStatus  SaveJPG(const char* pathname);

};




#endif // __STIMAGE_H__

