
#ifndef __STSCENENODE_H__
#define __STSCENENODE_H__

#include <stdlib.h>

#include "st.h"
#include "STTransform.h"

class STScene;


/**
* A node in the scene graph.  It may or may not contain a drawable object.  
* It may or may not contain child nodes.
* All nodes are associated with a STTransform
* The distinction between objects (which implement the STDrawable interface) and
* scene graph nodes allows scenes to use instancing.
*/ 
class STSceneNode {
	
	public:

		void		  Draw();

		STSceneNode*  PlaceObject(STDrawable* obj, STSceneNode* insertAfter=NULL);
		STSceneNode*  CreateNode(STSceneNode* insertAfter=NULL);
		void		  DeleteNode(STSceneNode* toRemove);

		STDrawable*	  GetObject();
		STSceneNode*  GetParent();
		STSceneNode*  GetPrev();
		STSceneNode*  GetNext();
		STSceneNode*  GetFirstChild();
		STSceneNode*  GetLastChild();
		int           GetNumChildren() const;

		const STTransform&  GetTransform();
		void				SetTransform(const STTransform& xform);

	private:

		STSceneNode(STDrawable* obj, STSceneNode* parent, STSceneNode* prev, STSceneNode* next);
		~STSceneNode();

		STDrawable*   mObj;

		STSceneNode*  mParentNode;
		STSceneNode*  mPrevNode;
		STSceneNode*  mNextNode;

		STSceneNode*  mChildrenHead;
		STSceneNode*  mChildrenTail;
		int			  mNumChildren;

		STTransform   mTransform;
		
		friend class STScene;
};


/**
* A scene graph.  All the real work is done in STSceneNode.
* The roll of this wrapped class is to preallocate and provide
* easy access to the root node of the scene graph.
*/
class STScene {

	public:
		STScene();
		~STScene();

		STSceneNode* GetRoot();
		void Draw();

	private:
		STSceneNode* mRoot;

};

#endif  // __STSCENENODE_H__



